# -*- coding: utf-8 -*-

# Standard Library
import datetime

# Vendor Library
from sqlalchemy import or_, and_

# Project Library
from kinder.core.declaration_status.models import DSS

from gisdo.algorithm import BaseApplicationAlgorithm
from gisdo.alchemy_models import (
    AlchemyWorkType, AlchemyDeclarationStatus,
    AlchemyChildren)
from gisdo.constants import short_day_types


class ApplicationCountByAge(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """Количество заявлений по возрастам"""

        return self._get_base_query(**kwargs)


class ApplicationCountByDelivery(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """По типу подачи: с портала или при личном обращении
        За исключением заявок в статусе Архивная"""

        portal = kwargs.get('portal', 0)
        base = self._get_base_query(**kwargs)

        return self.session.query(
            base
        ).join(
            AlchemyDeclarationStatus
        ).filter(
            and_(
                base.c.portal == portal,
                AlchemyDeclarationStatus.code != DSS.ARCHIVE
            )
        ).subquery()


class ApplicationCountByGroupType(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """Группы кратковременного пребывания"""
        base = self._get_base_query(**kwargs)
        return self.session.query(
            base
        ).join(
            AlchemyWorkType
        ).join(
            AlchemyDeclarationStatus
        ).filter(
            and_(
                AlchemyWorkType.code in short_day_types,
                AlchemyDeclarationStatus.code != DSS.ARCHIVE
            )
        ).subquery()


class ApplicationCountByDenied(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """Количество отказов"""

        base = self._get_base_query(**kwargs)
        return (
            self.session.query(
                base).join(
                    AlchemyDeclarationStatus)
            .filter(or_(AlchemyDeclarationStatus.code == DSS.DIDNT_COME,
                        AlchemyDeclarationStatus.code == DSS.REFUSED))
            .subquery())


class ApplicationWantChangeDou(BaseApplicationAlgorithm):

    @staticmethod
    def get_report_period():

        down_border = datetime.datetime.combine(
            datetime.date.min,
            datetime.datetime.min.time())
        up_border = datetime.datetime.combine(
            datetime.date.max,
            datetime.datetime.max.time())

        return down_border, up_border

    def get_result_set(self, **kwargs):
        """В статусе желает изменить ДОУ"""

        base = self._get_base_query(**kwargs)
        return (
            self.session.query(
                base).join(
                    AlchemyDeclarationStatus)
            .filter(AlchemyDeclarationStatus.code == DSS.WANT_CHANGE_DOU)
            .subquery())


class ApplicationWantChangeDouWithHN(ApplicationWantChangeDou):

    def get_result_set(self, **kwargs):
        """В статусе желает изменить ДОУ и потребностью по здоровью"""

        base = super(
            ApplicationWantChangeDouWithHN, self).get_result_set(
                **kwargs)
        return (
            self.session.query(base)
            .join(AlchemyChildren, AlchemyChildren.id == base.c.children_id)
            .filter(AlchemyChildren.health_need_id != None)
            .subquery())
